import com.imsl.stat.*;
import com.imsl.math.*;
import com.imsl.math.PrintMatrix;

public class RandomEx2 {

    static Random IMSLRandom() {
        Random r = new Random();
        r.setSeed(123457);
        r.setMultiplier(16807);
        return r;
    }

    public static void main(String args[]) throws com.imsl.IMSLException
    {
        double CorrMtrxIn[][] = {
            { 1.,                 -0.9486832980505138,  0.8164965809277261},
            {-0.9486832980505138,  1.,                 -0.6454972243679028},
            { 0.8164965809277261, -0.6454972243679028,  1.}
        };

        int nvar = 3;

        System.out.println("Random Example 2:");
        System.out.println("");

        for (int i = 0;  i < nvar;  i++) {
            for (int j = 0;  j < i;  j++) {
                System.out.println("CorrMtrxIn["+i+"]["
                        +j+"] = " + CorrMtrxIn[i][j]);
            }
        }

        PrintMatrixFormat pmf = new PrintMatrixFormat();
        pmf.setNumberFormat(new java.text.DecimalFormat("0.000000000"));

        new PrintMatrix("Input Correlation Matrix: ").print(pmf, CorrMtrxIn);
        System.out.println("Correlation Matrices calculated from");
        System.out.println(" Gaussian Copula imprinted multivariate sequence:");
        System.out.println("");

        // Compute the Cholesky factorization of CorrMtrxIn
        Cholesky CholMtrx = new Cholesky(CorrMtrxIn);

        for (int kmax = 500; kmax < 1000000; kmax *= 10) {

        System.out.println("# vectors in multivariate sequence: "+ kmax);

        double GCvart[][] = new double[kmax][];
        double GCdevt[][] = new double[kmax][nvar];
        Random r = IMSLRandom();
        for (int k = 0;  k < kmax;  k++) {
            GCvart[k] = r.nextGaussianCopula(nvar, CholMtrx);  //probs
    	    for (int j = 0;  j < nvar;  j++) {
/*
 *  invert Gaussian Copula probabilities to deviates using variable-specific
 *  inversions:  j = 0: Chi Square; 1: F; 2: Normal(0,1);
 *  will end up with deviate sequences ready for mapping to
 *  canonical correlation matrix:
 */
                if (j == 0) {
                    //convert probs into ChiSquare(df=10) deviates:
                    GCdevt[k][j] = InvCdf.chi(GCvart[k][j], 10.);
                }
                else if (j == 1) {
                    //convert probs into F(dfn=15,dfd=10) deviates:
                    GCdevt[k][j] = InvCdf.F(GCvart[k][j], 15., 10.);
                }
                else {
                    //convert probs into Normal(mean=0,variance=1) deviates:
                    GCdevt[k][j] = InvCdf.normal(GCvart[k][j]);
                }
            }
        }
/*
 *  extract Canonical Correlation matrix from arbitrarily distributed 
 *  deviate sequences GCdevt[k=0..kmax-1][j=0..nvar-1] which have been
 *  imprinted with CorrMtrxIn[i=1..nvar][j=1..nvar] above:
 */
        double CorrMtrx[][] = Random.canonicalCorrelation(GCdevt);
        double relerr;
        for (int i = 0;  i < nvar;  i++) {
            for (int j = 0;  j < i;  j++) {
                relerr = Math.abs(1. - (CorrMtrx[i][j]/CorrMtrxIn[i][j]));
                System.out.println("CorrMtrx["+i+"]["+j+"] = " + CorrMtrx[i][j]
                        + "; relerr = " + relerr);
            }
        }
        new PrintMatrix("Correlation Matrix: ").print(pmf, CorrMtrx);
        }
    }
}
